#include <millisDelay.h>

// Define the pins for the Ultrasonic sensor and the speaker
const int trigPin = 4;
const int echoPin = 3;
const int speakerPin = 2;

// Define the variables to store the duration of the pulse and the status of the echo pin
unsigned long pulseDuration;  // Duration of the pulse in microseconds
millisDelay repeatTimer;      // A delay at the end of each cycle to give the sonar sensor time to react.
millisDelay intervalTimer;    // A delay at the end of each cycle to create a gap in between the sounds.

void setup() {
  // Initialize serial communication
  Serial.begin(9600);

  // Set pin modes
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(speakerPin, OUTPUT);

  // Start the timers
  repeatTimer.start(100);
  intervalTimer.start(300);
}

void loop() {
  // Wait for the timer to complete before checking the distance and playing a sound.
  if (!repeatTimer.justFinished())
    return;

  // Send a 10us pulse to the trigger pin to start the measurement
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  // Wait for echo pulse to be received
  pulseDuration = pulseIn(echoPin, HIGH);

  if (intervalTimer.justFinished()) {
    // Calculate the distance in centimeters using the speed of sound in air (29.1 microseconds per centimeter)
    float distance = pulseDuration / 29.1 / 2;

    // Play audio based on the distance of the pulse
    if (distance > 100) {
      digitalWrite(speakerPin, LOW);  // No obstacles ahead
      noTone(speakerPin);             // Stop playing audio
    } else if (distance > 75) {
      tone(speakerPin, 400);  // Play audio with a sertain frequency
    } else if (distance > 25) {
      tone(speakerPin, 800);  // Play audio with a sertain frequency
    } else if (distance > 0) {
      tone(speakerPin, 1000);  // Play audio with a sertain frequency
    }

    // Print distance and frequency to serial monitor
    Serial.print("Distance: ");
    Serial.print(distance);
    Serial.println(" cm");

    // Restart the interval timer
    intervalTimer.repeat();
  } else {
    // Stop playing audio
    noTone(speakerPin);
  }

  // Repeat the small delay to give the sonar sensor time to react and the buzzer sound to be heard.
  repeatTimer.repeat();
}